Disabilitazione interruzioni dopo lettura di RBR
================================================


0. Preparazione
~~~~~~~~~~~~~~~

Usare utente1.cpp per il modulo utente:
```
    ln -fs utente1.cpp utente/utente.cpp
```
Il programma contenuto in utente1.cpp è basato sull'esempio mailbox-dyn.cpp.
Il programma crea due processi scrittori (pms) e un processo lettore (pml). I
processi scrittori inviano messaggi tramite una mailbox (della capienza di un
solo messaggio) al processo lettore, che li stampa sul video. Tutti i messaggi
sono grandi 80 byte e ciascun messaggio è ottenuto tramite `ceread_n()`
sull'intefaccia CE numero 0.  I processi scrittori leggono il messaggio in un
buffer temporaneo (tmpbuf[0] per il primo e tmpbuf[1] per il secondo) e poi lo
copiano nella mailbox quando acquisiscono l'accesso.


1. Introduzione dell'errore
~~~~~~~~~~~~~~~~~~~~~~~~~~~

Modificare `c_driver_ce` spostando la lettura di RBR all'inizio della funzione.
Questo causa il problema descritto al punto 1 in fondo alla pagina 6 della
dispensa `io-driver.pdf`.

Compilare e avviare con `boot -C` (l'opzione `-C` serve a non colorare l'output
ed evitare che alcuni messaggi di log non appaiano tempestivamente).  Sul
monitor della macchina QEMU dovrebbe apparire

```
messaggio 0 da 0:
ABCDEFGHIJKLMNOPQRSTUVWXYZABCDEFGHIJKLMNOPQRSTUVWXYZABCDEFGHIJKLMNOPQRSTUVWXYZAB
```

mentre sul log dovrebbe apparire

```
WRN     7       semaforo errato: 70
WRN     7       proc 7: corpo pms(unsigned long) [utente.cpp:27](1), livello UTENTE, precedenza 5
WRN     7         RIP=c_abort_p [sistema.cpp:1224] CPL=LIV_SISTEMA
WRN     7         RFLAGS=2 [-- -- -- -- -- -- -- -- -- --, IOPL=SISTEMA]
WRN     7         RAX=            14e0 RBX=               0 RCX=      fffffffc80 RDX=             3f8
WRN     7         RDI=               1 RSI=             3f8 RBP=      ffffffff98 RSP=      ffffffff78
WRN     7         R8 =               0 R9 =               0 R10=               0 R11=               0
WRN     7         R12=               0 R13=               0 R14=               0 R15=               0
WRN     7         backtrace:
WRN     7         > c_sem_signal [sistema.cpp:356]
WRN     7         > a_sem_signal [sistema.s:339]
WRN     7         > sem_signal [utente.s:48]
WRN     7         > pms(unsigned long) [utente.cpp:33 (discriminator 3)]
WRN     7       Processo 7 abortito
```


2. Analisi del messaggio di errore
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Il processo 7, che stava eseguendo `pms(1)`, è stato terminato forzatamente
perché ha tentato di usare un semaforo non valido (il numero 70) in una
`sem_signal`.  La `sem_signal` era stata invocata alla riga 33 di utente.cpp. A
quella riga troviamo
```
        sem_signal(mailbox_piena);
```
ma provando a stampare il valore di mailbox_piena quando il semaforo viene
allocato (riga 65) vediamo che dovrebbe essere 1. Il programma utente non
contiene altri assegnamenti a mailbox_piena: come è potuto accadere che
mailbox_piena sia diventato 70?


3. Esecuzione con il debugger (1)
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Avviamo il sistema con `boot -C -g` e colleghiamoci con `debug`. Nel debugger
usiamo il comando
```
watch mailbox_piena
```
per fermare l'esecuzione ogni volta che mailbox_piena cambia valore. Lasciamo
proseguire l'esecuzione con `c`. L'esecuzione si ferma una prima volta subito
dopo la linea
```
    mailbox_piena = sem_ini(0);
```
in utente.cpp, che è quella che ci aspettiamo. Facciamo proseguire l'esecuzione
con `c` e osserviamo che si ferma una seconda volta, subito dopo l'istruzione
```
    *ce->buf = b;
```
in c_driver_ce in sistema.cpp. Lo stato mostrato dal debugger conferma che
ce->buf sta puntando a mailbox_piena. Un'altra cosa che possiamo notare è che
il valore di ce->quanti non è corretto: c_driver_ce sta gestendo un interrupt
in più rispetto a quelli richiesti. Osservando gli indirizzi di mailbox_piena
e di tmpbuf, vediamo che distano di 160 byte (il compilatore ha allocato
mailbox_piena subito dopo tmpbuf). Il primo processo scrittore usa tmpbuf[0]
e il secondo usa tmpbuf[1], che si trova a 80 byte dall'inizio di tmpbuf. Se
l'operazione `ceread_n()` avviata dal secondo scrittore legge più di 80 byte,
i byte in eccesso andranno a sovrascrivere mailbox_piena, e questo è quello
che è avvenuto. Il processo utente, però, ha richiesto proprio 80 byte e non
di più (riga 29 di utente.cpp), quindi ci deve essere un errore in c_driver_ce.


4. Esecuzione con il debugger (2)
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Chiudiamo la sessione di debug e avviamone un'altra che riparta da zero, per
fermarci ad un passo prima che l'errore si verifichi. Questa volta mettiamo un
breakpoint sull'istruzione che legge RBP, ma facciamo fermare l'esecuzione solo
quando `ce->quanti == 1`:
```
break sistema.cpp:xxx if ce->quanti == 1
```
dove xxx è il numero di riga di `char b = inputb(ce->iRBR)` in sistema.cpp. Facciamo
proseguire l'esecuzione con `c`. Quando l'esecuzione si ferma, potrebbe essere
durante l'operazione di I/O del primo o del secondo scrittore. Ce ne accorgiamo
dal valore di ce->buf: se ce->buf punta a tmpbuf+79, si tratta del primo scrittore.
A noi interessa il secondo scrittore, quindi proseguiamo con `c`. Questa volta
dovremmo trovare che c->buf punta a tmpbuf+159, cioè un passo prima di puntare
a mailbox_piena. Controlliamo lo stato dell'apic con
```
apic
```
Dovremmo vedere che il registro ISR dell'APIC registra, correttamente, che è in
servizio l'interruzione di tipo 0xa0, mentre il registro IRR è tutto a zero.
Ora facciamo eseguire l'istruzione `char b = inputb(ce->iRBR)` con il comando
`n` e poi rieseguiamo il comando `apic`. Ora dovremmo vedere che il registro
IRR contiene una nuova richiesta di tipo 0xa0, proveniente quindi dall'interfaccia
CE. Quello che è accaduto è che l'interfaccia aveva un nuovo dato disponibile, oltre
quello appena letto.
Siccome l'interfaccia era ancora abilitata a generare richieste di
interruzione, la lettura di RBR l'ha autorizzata a inviarne subito una nuova.
Questo ulteriore byte non era stato richiesto dall'utente, e infatti
il buffer tmpbuf non ha spazio per contenerlo. Ma ormai l'interfaccia ha fatto la
richiesta e l'APIC l'ha registrata. Se facciamo proseguire l'esecuzione fino
al punto in cui disabilitiamo l'interfaccia a generare richieste di interruzione
(istruzione `outputb(0, ce->iCTL)`) vediamo che questa cosa non influisce sullo
stato dell'APIC. La richiesta in più verrà dunque inoltrata e poi accettata dalla CPU
(dopo l'invio dell'EOI e la riabilitazione delle interruzioni esterne mascherabili
per effetto della iretq che termina il driver), come possiamo verificare inserendo
un breakpoint all'inizio di `a_driver_ce_0` e facendo proseguire l'esecuzione.


5. Soluzione
~~~~~~~~~~~~

La soluzione, ovviamente, consiste nel rimettere la `char b = inputb(ce->iRBR)`
nella posizione originaria, in modo che sia *successiva* alla disabilitazione
delle interruzioni quando ce->quanti raggiunge 0. Ogni volta che eseguiamo una
azione che funge da risposta alle richieste di interruzione da parte di una
interfaccia, dobbiamo chiederci se possiamo tollerare una nuova richiesta
di interruzione e, in caso contrario, disabilitarla preventivamente.



Mancato controllo degli argomenti
=================================


0. Preparazione
~~~~~~~~~~~~~~~

Usare il programma utente2.cpp:
```
    ln -fs utente2.cpp utente/utente.cpp
```
Questo programma crea due processi: buono e cattivo. Il processo buono
stampa `tutto OK` ogni 5 secondi; il processo cattivo vuole causare danni
al processo buono (per esempio causando lo shutdown dell'intero sistema).

Normalmente, il processo cattivo non dovrebbe riuscire nel suo intento: il
processo buono usa soltanto variabili private (la variabile `i`, allocata sulla
sua pila e dunque nel suo spazio utente/privato) e ha una priorità maggiore,
quindi il cattivo non dovrebbe aver modo di influenzarne l'esecuzione.  A puro
titolo di esempio, nel programma iniziale il processo cattivo esegue una
divisione per zero, che causa dunque una eccezione di divisione per zero (tipo
0, prima entrata della IDT). Il modulo sistema, però, intercetta tutte le
eccezioni e causa la terminazione solo del processo che l'ha causata: se
compiliamo (`compile`) e avviamo il sistema (`boot -C`) vediamo che il processo
cattivo viene abortito, ma il processo buono continua a stampare i propri
messaggi indisturbato, come ci aspettiamo.

Tutto ciò, però, è verso solo se non ci sono bug nel modulo sistema. Proviamo
quindi ad introdurne uno per vedere come cambiano le cose.


1. Introduzione dell'errore
~~~~~~~~~~~~~~~~~~~~~~~~~~~

Commentare l'invocazione di `c_access` in `c_ceread_n`. A questo punto
il processo cattivo può passare indirizzi arbitrari alla `ceread_n()`.
Proviamo vari casi.


2. Passaggio di indirizzi appartenenti al modulo sistema
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Il cattivo decide di sovrascrivere la prima entrata della IDT, in modo che
generando una eccezione di divisione per zero si causi una eccezione nel modulo
sistema stesso. Le eccezioni generate da istruzioni del modulo sistema non sono
in genere recuperabili e, se si verificano, il sistema è costretto ad eseguire
uno shutdown forzato (panic), terminando quindi anche il processo buono.

Trovare l'indirizzo della IDT non è un problema, se il cattivo ha accesso
al modulo sistema compilato:
```
    nm buiild/sistema | grep idt
```
(il comando `nm` stampa i simboli contenuti nel modulo sistema e `grep` cerca la
stringa idt; la cosa funziona perché abbiamo non abbiamo eliminato la tabella
dei simboli dal modulo sistema, ma anche in assenza di quella ci sarebbero altri
modi). Assumiamo che l'indirizzo della IDT sia 0x230680. Ovviamente il cattivo
non può accedervi direttamente: se proviamo a modificarlo nel modo seguente:
```
void cattivo(natq)
{
    char *idt = (char*)0x230680;
    // sovrascriviamo un paio di byte della prima entrata con valori a caso
    idt[0] = 'A'; idt[1] = 'B';
    terminate_p();
}
```
Il programma compila correttamente, ma quando avviamo il sistema vediamo che
l'istruzione `idt[0] = 'A'` genera una eccezione di page fault e il
modulo sistema, di nuovo, abortisce il processo cattivo senza influenzare il
processo buono. Il page fault è stato sollevato dalla MMU, che ha impedito
l'accesso ad indirizzi di sistema (bit U/S non settato) mentre il processore
si trovava a livello utente.

Proviamo però a modificare il cattivo come segue:
```
void cattivo(natq)
{
    ceread_n(0, (char*)0x230680, 2);
    foo(1, 0);
    terminate_p();
}
```
In questo caso il cattivo sta cercando di sovrascrivere i primi due byte
della IDT non direttamente, ma indirettamente tramite la `ceread_n()`.
Anche questo programma compila correttamente ma in più, quando avviamo
il sistema, causa il system panic a cui il cattivo mirava. Cosa è successo?
Questo è un esempio del problema del Cavallo di Troia: in questo caso il
cavallo di Troia è l'indirizzo 0x230680 e le mura di Troia sono rappresentate
dalla protezione hardware, con il gate della `ceread_n()` che funge da cancello da
attraversare. Il cattivo fa passare il cavallo attraverso il cancello
eseguendo delle operazioni che l'hardware (CPU e MMU) non riconoscono
come malevole: per passare l'indirizzo esegue qualcosa come
```
    movl $0x230680, %esi
```
che la CPU non è in grado di distinguere da una qualsiasi altra istruzione che
carichi una costante in un registro. La MMU, poi, non è nemmeno coinvolta
nell'esecuzione di questa istruzione, in quanto non prevede accessi in memoria.
Il cavallo attraversa dunque il cancello, la `ceread_n()` (dal momento che
abbiamo commentato i controlli della `c_access`) lo scrive in ce->buf e il
driver lo usa per scrivervi i byte letti dalla periferica. La MMU permette
questa scrittura, perché il driver gira a livello sistema.  L'effetto
complessivo è che, quando la `ceread_n()` termina, il processo cattivo sa che
l'entrata 0 della IDT è stata sovrascritta con valori casuali.  Il cattivo
causa dunque volontariamente una eccezione di divisione per zero: la CPU genera
l'eccezione di tipo 0 e salta all'indirizzo (corrotto) contenuto in idt[0]; a
quel punto inizia ad eseguire istruzioni imprevedibili e presto o tardi causa
un qualche errore che, essendosi verificato mentre l'instruction pointer si
trovava comunque nella parte sistema/privata, viene considerato fatale.

3. Esecuzione nel debugger
~~~~~~~~~~~~~~~~~~~~~~~~~~

Riavviamo il sistema con `boot -g` e colleghiamoci con il debugger (`debug`).
Inseriamo un breakpoint alla riga di `cattivo` che contiene l'invocazione
di `ceread_n()` e lasciamo proseguire l'esecuzione (`c`). Eseguiamo passo passo
(comando `s`) per verificare che si riesce ad arrivare fino alla `c_ceread_n`
senza alcun errore. Eseguendo con `n` (per evitare di eseguire passo passo anche
`sem_wait`) arrivamo al punto in cui viene assegnato `ce->buf` per vericare
che effettivamente ora punta alla IDT. Mettiamo ora un breakpoint sull'istruzione
di `c_driver_ce` che scrive in `ce->buf` e lasciamo prosegure l'esecuzione (`c`).
Quando l'esecuzione si ferma nuovamente, controlliamo lo stato dell'entrata 0
della IDT prima e dopo l'assegnamento, con
```
    idt 0
```
Prima dell'assegnamento dovremmo vedere qualcosa del genere:
```
[ 0]: sys intr           200efb (sistema:exc_div_error)
```
L'entrata punta ancora correttamente alla routine `exc_div_error` del modulo
sistema. Lasciamo eseguire l'assegnamento con `n` e notiamo che non causa
errori (siamo a livello sistema e abbiamo il diritto di scrivere a
quell'indirizzo).  Dopo l'assegnamento eseguiamo di nuovo `idt` e dovremmo
vedere che il byte meno significativo dell'indirizzo è cambiato. Ora l'entrata
0 della IDT è stata corrotta, ma questo non causa immediatamente un errore: lo
causerà quando il processore tenterà di attraversare quell'entrata.  Ora
torniamo al processo utente inserendo un breakpoint nella funzione foo e
lasciando proseguire l'esecuzione
```
    b foo
    c
    c
```
(la prima volta l'esecuzione si fermerà di nuovo nel driver, in quanto il
cattivo ha chiesto di leggere due byte). A questo punto eseguiamo `si` un po'
di volte per eseguire singole istruzioni di linguaggio macchina, fino a quando
non si arriva all'istruzione `idivl` che solleva l'eccezione 0. Notiamo che il
gate viene correttamente e l'esecuzione passa al livello sistema, ma in un
punto scorretto (nel mio caso in mezzo alla funzione `process_dump`). Anche
questo non è ancora un errore e l'esecuzione può proseguire ancora per un po'
(a dimostrazione che la CPU non ha idea di cosa gli si chieda di fare e, purché
le singole istruzioni che le chiediamo di eseguire siano valide, le esegue), ma
prima o poi causerà un errore fatale (nel mio caso all'interno di una di una
`strnlen` che avrebbe dovuto ricevere come argomento una stringa, ma invece
riceve un puntatore nullo).


4. Ulteriori esempi
~~~~~~~~~~~~~~~~~~~

Si deve stare molto attenti a controllare i parametri delle primitive,
permettendo solo i casi sicuramente validi. Nel caso di una primitiva come
cerea_n(), tutti gli indirizzi in [buf,buf+quanti) devono:

- appartenere alle parti utente;
- essere normalizzati;
- se il sistema vi deve scrivere, devono anche essere accessibili in scrittura;

Come esempi, lasciando sempre commentata l'invocazione di `c_access` in
`c_ceread_n`, provare a modificare il programma cattivo in modo che passi alla
`ceread_n()` l'indirizzo 0xaaaa000000000000 (non normalizzato) o l'indirizzo
`&foo` (non scrivibile). In entrambi i casi l'indirizzo sarà accettato dalla
`ceread_n()` e causerà un panic (questa volta immediato) non appena il driver
tenterà di scrivervi.



Controlli insufficienti nei parametri
=====================================

0. Preparazione
~~~~~~~~~~~~~~~

Usare il programma utente3.cpp:
```
    ln -fs utente3.cpp utente/utente.cpp
```
Il programma è basato su utente2.cpp, con alcune modifiche che permettono di
evidenziare più facilmente il problema di cui si vuole parlare. In particolare,
il processo buono non esegue delay() e gira ad una priorità inferiore a quella
del processo cattivo. Questo rende altamente probabile il fatto che il driver
associato all'operazione di I/O iniziata dal processo cattivo interrompa il
processo buono. Il processo cattivo esegue una delay(1) all'inizio, per dare il
tempo al processo buono di eseguire almeno l'inizializzazione della sua
variabile `i` prima che parta l'operazione di I/O.

Il processo cattivo passa alla `ceread_n()` l'indirizzo di una propria variable
locale, allocata sulla sua pila utente e dunque appartenente alla parte
utente/privata del suo spazio di indirizzamento.


1. Introduzione dell'errore
~~~~~~~~~~~~~~~~~~~~~~~~~~~

Reintrodurre l'invocazione di `c_access` in `c_ceread_n`, se era stata
commentata, ma modificarla nel seguente modo
```
	if (!c_access(reinterpret_cast<vaddr>(buf), quanti, true, false)) {
```
L'ulteriore parametro `false` chiede a `c_access` di non controllare
che [buf,buf+quanti) cada all'interno della parte utente/condivisa.
In questo modo, l'indirizzo `&myvar` passato dal cattivo verrà accettato
dalla `ceread_n()`.

Se ora compiliamo e poi avviamo il sistema, vedremo che il valore della
variabile `i` stampata dal processo buono cambia improvvisamente valore,
passando da un valore basso ad un valore molto alto tra una stampa e
la successiva. Cosa è accaduto?


2. Analisi dell'errore
~~~~~~~~~~~~~~~~~~~~~~

Il processo cattivo ha passato alla `ceread_n()` un indirizzo *virtuale*
appartenente alla propria parte utente/privata. Gli indirizzi virtuali di
questa parte hanno una traduzione diversa in ogni processo.  La funzione
`c_ceread_n` lo ha preso così come era e lo ha copiato in `ce->buf`.
L'indirizzo è poi stato usato dal driver nell'istruzione `*ce->buf = b`, e qui
è nato il problema: in quel momento il TRIE attivo era quello del processo
interrotto, che in questo caso era il processo buono.  La MMU ha dunque
tradotto l'indirizzo &myvar usando il TRIE del processo buono, sovrascrivendo
la sua pila utente. In questo esempio la cosa è immediatamente visibile, in
quanto la variabile `myvar` del cattivo e la variabile `i` del buono hanno lo
stesso indirizzo virtuale. Il cattivo è dunque riuscito a influenzare l'esecuzione
del buono, anche se questo stava usando esclusivamente variabili private.
Di nuovo, ciò è stato possibile a causa di un bug nel sistema (in questo caso,
il fatto di aver usato direttamente l'indirizzo virtuale passato dall'utente,
senza nemmeno controllare che appartenesse alla parte utente/condivisa).


3. Esecuzione nel debugger
~~~~~~~~~~~~~~~~~~~~~~~~~~

Avviamo il sistema con `boot -c` e colleghiamoci con il debugger. Inseriamo
dei breakpoint in `cattivo`, `buono`, `c_ceread_n` e in `c_driver_ce`:
```
    b cattivo
    b buono
    b c_ceread_n
    b c_driver_c
```
quindi lasciamo proseguire l'esecuzione con `c`. 
L'esecuzione si ferma all'inizio del cattivo. Stampiamo l'indirizzo di myavar:
```
    p &myvar
```
Prendiamo nota del contenuto di cr3 (nella riga '[protezione]', supponiamo sia
0x4af000) e poi proseguiamo con `c`. Ora l'esecuzione si ferma all'inizio del
buono.  Eseguiamo un paio di passi (per esempio con `n`) in modo da entrare nel
ciclo for che definisce la variabile `i` e quindi stampiamone l'indirizzo
```
    p &i
```
Osserviamo che è lo stesso di &myavar. Le due variabili, però, non coincidono
perché i due indirizzi vengono tradotti diversamente nei due processi. Possiamo
vedere il percorso di traduzione di &i nel processo corrente (il buono) con
```
    vm path &i
```
ottenendo qualcosa del genere:
```
cr3: 0x00475000
777: WU--A--  ➞ 0x0049c000
          777: WU--A--  ➞ 0x0049d000
                    777: WU--A--  ➞ 0x0049e000
                              777: WU--AD-  ➞ 0x004ae000
```
La stampa mostra che il percorso passa dalle entrate 777 (in ottale) di quattro
tabelle partendo da quella puntata da cr3 e la sua traduzione cade nel frame
0x4ae000.  Sempre restando con questo processo attivo, possiamo osservare
anche il percorso di traduzione dello stesso indirizzo partendo da un altro
cr3, come quello del processo cattivo, scrivendo:
```
    vm path &i @ 0x4af000
```
dove 0x4af000 è l'indirizzo contenuto nel cr3 del processo cattivo, di cui
avevamo preso nota prima. Dovremmo ottenere qualcosa del genere:
```
cr3: 0x004af000
777: WU--A--  ➞ 0x004b4000
          777: WU--A--  ➞ 0x004b5000
                    777: WU--A--  ➞ 0x004b6000
                              777: WU--AD-  ➞ 0x004c6000
```
Vediamo come, partendo da 0x4af000, la traduzione dello stesso indirizzo cada
nel frame 0x4c6000 invece che nel frame 0x4ae000.  Lasciamo ora proseguire
l'esecuzione, che si fermerà in `c_ceread_n`.  Osserviamo che il parametro
`buf` contiene l'indirizzo virtuale &myvar==&i e lo scrive inalterato in
`ce->buf`. Facciamo proseguire l'esecuzione l'ultima volta, arrivando
all'inizio di `c_driver_ce`. Osserviamo che la variabile `esecuzione` punta al
processo buono e in cr3 c'è l'indirizzo della tabella radice del processo buono
(0x475000 nel mio caso).  La scrittura in `ce->buf` sovrascriverà dunque il
valore della variabile `i` del processo buono. Possiamo verificarlo proseguendo
l'esecuzione con `n` fino alla `iretq`, a quel punto usiamo il comando `s` per
eseguire solo la `iretq` e tornare al processo buono. Qui stampiamo il nuovo
valore di `i` e confrontiamolo con l'ultimo che il processo buono aveva
mostrato sul monitor della macchina QEMU. Si noti che la sovrascrittura di `i`
proseguirà anche per gli altri 3 byte che il cattivo aveva chiesto di leggere
dalla periferica CE. Visto che la periferica restituisce codici ASCII in sequenza
a partire da 'A', il valore finale di 'i' sarà 0x44434341 == 1145258561.
